@@ -292,7 +292,83 @@ class Sequential():
             return 
         else:
             raise ValueError("unknown value for verbose")
-        
+    
+    def live_stats(self,live_stats, figure):
+        import matplotlib.pyplot as plt
+        if live_stats == 1:
+            figure.clear()
+            
+            plt.subplot(1,2,1)
+            plt.title("Loss Evolution")
+            plt.xlabel("Epoch")
+            plt.ylabel("Loss")
+            plt.plot(self.train_loss_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_loss_records[-1]), 3)}")
+            if self.val_inputs is not None and self.val_targets is not None:
+                plt.plot(self.val_loss_records, color="orange", label=f"validation dataset: {round(float(self.val_loss_records[-1]), 3)}")
+            plt.legend()
+            
+            plt.subplot(1,2,2)
+            plt.title("Accuracy Evolution")
+            plt.xlabel("Epoch")
+            plt.ylabel("Accuracy")
+            plt.plot(self.train_accuracy_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_accuracy_records[-1]), 3)}")
+            if self.val_inputs is not None and self.val_targets is not None:
+                plt.plot(self.val_accuracy_records, color="orange", label=f"validation dataset: {round(float(self.val_accuracy_records[-1]), 3)}")
+            plt.legend()
+            
+            figure.canvas.draw()
+            figure.canvas.flush_events()
+            
+        elif live_stats == 2:
+            figure.clear()
+            
+            plt.subplot(1,1,1)
+            if len(self.train_inputs[0,:]) <= 2 and len(self.train_targets[0,:]) == 1:
+                X_set, y_set = self.train_inputs, self.train_targets
+                X1, X2 = np.meshgrid(np.arange(start = X_set[:, 0].min() - 1, stop = X_set[:, 0].max() + 1, step = 0.01),
+                                     np.arange(start = X_set[:, 1].min() - 1, stop = X_set[:, 1].max() + 1, step = 0.01))
+                plt.contourf(X1, X2, self.predict(np.array([X1.ravel(), X2.ravel()]).T).reshape(X1.shape),
+                             alpha = 0.3, cmap = "coolwarm")
+                
+            plt.scatter(self.train_inputs[:,0], self.train_inputs[:,1], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            
+            figure.canvas.draw()
+            figure.canvas.flush_events()
+            
+        elif live_stats == -1:
+            figure.clear()
+            
+            plt.subplot(1, 2, 2)
+            if len(self.train_inputs[0,:]) <= 2 and len(self.train_targets[0,:]) == 1:
+                X_set, y_set = self.train_inputs, self.train_targets
+                X1, X2 = np.meshgrid(np.arange(start = X_set[:, 0].min() - 1, stop = X_set[:, 0].max() + 1, step = 0.01),
+                                     np.arange(start = X_set[:, 1].min() - 1, stop = X_set[:, 1].max() + 1, step = 0.01))
+                plt.contourf(X1, X2, self.predict(np.array([X1.ravel(), X2.ravel()]).T).reshape(X1.shape),
+                             alpha = 0.3, cmap = "coolwarm")
+                
+            plt.scatter(self.train_inputs[:,0], self.train_inputs[:,1], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
+            
+            plt.subplot(2,2,1)
+            plt.title("Loss Evolution")
+            plt.xlabel("Epoch")
+            plt.ylabel("Loss")
+            plt.plot(self.train_loss_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_loss_records[-1]), 3)}")
+            if self.val_inputs is not None and self.val_targets is not None:
+                plt.plot(self.val_loss_records, color="orange", label=f"validation dataset: {round(float(self.val_loss_records[-1]), 3)}")
+            plt.legend()
+            
+            plt.subplot(2,2,3)
+            plt.title("Accuracy Evolution")
+            plt.xlabel("Epoch")
+            plt.ylabel("Accuracy")
+            plt.plot(self.train_accuracy_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_accuracy_records[-1]), 3)}")
+            if self.val_inputs is not None and self.val_targets is not None:
+                plt.plot(self.val_accuracy_records, color="orange", label=f"validation dataset: {round(float(self.val_accuracy_records[-1]), 3)}")
+            plt.legend()
+            
+            figure.canvas.draw()
+            figure.canvas.flush_events()
+    
     def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = 0):
         for n in [-1,0,1,2]:
             if live_stats == n:
@@ -322,16 +398,12 @@ class Sequential():
                 
         if live_stats == -1:
             update = 50
+            figure = plt.figure()
         elif live_stats == 0:
             update = 1
         elif live_stats == 1 or live_stats == 2:
             update = 25
-
-        if live_stats != 0:
-            if live_stats == 1 or live_stats == -1:
-                figure1,axs1 = plt.subplots(1,2)
-            if live_stats == 2 or live_stats == -1:
-                figure2,axs2 = plt.subplots(1,1)
+            figure = plt.figure()
             
         if verbose == 1:
             for epoch in tqdm(range(epochs)):
@@ -347,44 +419,8 @@ class Sequential():
                 
                 if live_stats != 0:
                     if epoch % update == 0:
-                        if live_stats == 1 or live_stats == -1:
-                            axs1[0].cla()
-                            axs1[1].cla()
+                        self.live_stats(live_stats, figure)
                             
-                            axs1[0].set_title("Loss Evolution")
-                            axs1[0].set_xlabel("Epoch")
-                            axs1[0].set_ylabel("Loss")
-                            axs1[1].set_title("Accuracy Evolution")
-                            axs1[1].set_xlabel("Epoch")
-                            axs1[1].set_ylabel("Accuracy")
-                            
-                            axs1[0].plot(self.train_loss_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_loss_records[-1]), 3)}")
-                            axs1[1].plot(self.train_accuracy_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_accuracy_records[-1]), 3)}")
-                            if self.val_inputs is not None and self.val_targets is not None:
-                                axs1[0].plot(self.val_loss_records, color="orange", label=f"validation dataset: {round(float(self.val_loss_records[-1]), 3)}")
-                                axs1[1].plot(self.val_accuracy_records, color="orange", label=f"validation dataset: {round(float(self.val_accuracy_records[-1]), 3)}")
-                                
-                            axs1[0].legend()
-                            axs1[1].legend()
-                            
-                            figure1.canvas.draw()
-                            figure1.canvas.flush_events()
-                            
-                        if live_stats == 2 or live_stats == -1:
-                            axs2.cla()
-                            
-                            if len(self.train_inputs[0,:]) <= 2 and len(self.train_targets[0,:]) == 1:
-                                X_set, y_set = self.train_inputs, self.train_targets
-                                X1, X2 = np.meshgrid(np.arange(start = X_set[:, 0].min() - 1, stop = X_set[:, 0].max() + 1, step = 0.01),
-                                                     np.arange(start = X_set[:, 1].min() - 1, stop = X_set[:, 1].max() + 1, step = 0.01))
-                                axs2.contourf(X1, X2, self.predict(np.array([X1.ravel(), X2.ravel()]).T).reshape(X1.shape),
-                                             alpha = 0.3, cmap = "coolwarm")
-                                
-                            axs2.scatter(self.train_inputs[:,0], self.train_inputs[:,1], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
-                            
-                            figure2.canvas.draw()
-                            figure2.canvas.flush_events()
-                
                 self.verbose(verbose, epoch, epochs, start)
         else:
             for epoch in range(epochs):
@@ -400,43 +436,7 @@ class Sequential():
                 
                 if live_stats != 0:
                     if epoch % update == 0:
-                        if live_stats == 1 or live_stats == -1:
-                            axs1[0].cla()
-                            axs1[1].cla()
-                            
-                            axs1[0].set_title("Loss Evolution")
-                            axs1[0].set_xlabel("Epoch")
-                            axs1[0].set_ylabel("Loss")
-                            axs1[1].set_title("Accuracy Evolution")
-                            axs1[1].set_xlabel("Epoch")
-                            axs1[1].set_ylabel("Accuracy")
-                            
-                            axs1[0].plot(self.train_loss_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_loss_records[-1]), 3)}")
-                            axs1[1].plot(self.train_accuracy_records, color="#1f77b4", label=f"training dataset: {round(float(self.train_accuracy_records[-1]), 3)}")
-                            if self.val_inputs is not None and self.val_targets is not None:
-                                axs1[0].plot(self.val_loss_records, color="orange", label=f"validation dataset: {round(float(self.val_loss_records[-1]), 3)}")
-                                axs1[1].plot(self.val_accuracy_records, color="orange", label=f"validation dataset: {round(float(self.val_accuracy_records[-1]), 3)}")
-                            
-                            axs1[0].legend()
-                            axs1[1].legend()
-                            
-                            figure1.canvas.draw()
-                            figure1.canvas.flush_events()
-                        
-                        if live_stats == 2 or live_stats == -1:
-                            axs2.cla()
-                            
-                            if len(self.train_inputs[0,:]) <= 2 and len(self.train_targets[0,:]) == 1:
-                                X_set, y_set = self.train_inputs, self.train_targets
-                                X1, X2 = np.meshgrid(np.arange(start = X_set[:, 0].min() - 1, stop = X_set[:, 0].max() + 1, step = 0.01),
-                                                     np.arange(start = X_set[:, 1].min() - 1, stop = X_set[:, 1].max() + 1, step = 0.01))
-                                axs2.contourf(X1, X2, self.predict(np.array([X1.ravel(), X2.ravel()]).T).reshape(X1.shape),
-                                             alpha = 0.3, cmap = "coolwarm")
-                            
-                            axs2.scatter(self.train_inputs[:,0], self.train_inputs[:,1], c=self.train_outputs[:,:], cmap="coolwarm", alpha=1)
-                            
-                            figure2.canvas.draw()
-                            figure2.canvas.flush_events()
+                        self.live_stats(live_stats, figure)
                         
                 self.verbose(verbose, epoch, epochs, start)
         
@@ -444,24 +444,24 @@ class Sequential():
         if self.__is_trained__ == True:
             import matplotlib.pyplot as plt
             
-            figure1,axs1 = plt.subplots(1,2)
+            figure,axs = plt.subplots(1,2)
             
-            axs1[0].set_title("Loss Evolution")
-            axs1[0].set_xlabel("Epoch")
-            axs1[0].set_ylabel("Loss")
-            axs1[0].plot(self.train_loss_records, label="training dataset")
+            axs[0].set_title("Loss Evolution")
+            axs[0].set_xlabel("Epoch")
+            axs[0].set_ylabel("Loss")
+            axs[0].plot(self.train_loss_records, label="training dataset")
             if self.val_inputs is not None and self.val_targets is not None:
-                axs1[0].plot(self.val_loss_records, label="validation dataset")
+                axs[0].plot(self.val_loss_records, label="validation dataset")
             
-            axs1[1].set_title("Accuracy Evolution")
-            axs1[1].set_xlabel("Epoch")
-            axs1[1].set_ylabel("Accuracy")
-            axs1[1].plot(self.train_accuracy_records, label="training dataset")
+            axs[1].set_title("Accuracy Evolution")
+            axs[1].set_xlabel("Epoch")
+            axs[1].set_ylabel("Accuracy")
+            axs[1].plot(self.train_accuracy_records, label="training dataset")
             if self.val_inputs is not None and self.val_targets is not None:
-                axs1[1].plot(self.val_accuracy_records, label="validation dataset")
+                axs[1].plot(self.val_accuracy_records, label="validation dataset")
                 
-            axs1[0].legend()
-            axs1[1].legend()
+            axs[0].legend()
+            axs[1].legend()
             
             plt.show()
         else:
